from collections import OrderedDict
from typing import List

from pydantic import model_validator, ConfigDict
from pydantic.dataclasses import dataclass

from glider_client.command_results import GliderCommandResults, OptimizeAdcResults, SteppingPOIResults, \
    SteppingPOIResultWithAnalogBuffers, MockCommandResults
from glider_client.utils.mcu_registers import ANALOG_CHANNEL1, ANALOG_CHANNEL2, ADC_SAMPLING_TIME_DICT_INV, \
    ADC_SAMPLING_TIMES

from glider_client.utils.regexps import CamelCase_to_snake_case


@dataclass
class GliderCommandDataset:

    @classmethod
    def from_dict(cls, mydict):
        return cls(**mydict)

    def get_results_klass(self):
        return GliderCommandResults

    @classmethod
    def get_command_name(cls):
        return CamelCase_to_snake_case(cls.__name__).replace('_dataset', '')

    __pydantic_config__ = ConfigDict(extra='forbid')


@dataclass
class AdminSetSettingsDataset(GliderCommandDataset):
    cavity_index: int | None
    pulsing_mode: str| None = None
    applied_voltage: float | None = None
    pulse_period: float | None = None
    pulse_width: float | None = None
    current_limit: float | None = None
    stage_angle: float | None = None
    temperature: int | None = None
    pga1_gain: str | None = None
    pga2_gain: str | None = None
    max_current: float | None = None
    max_voltage: float | None = None
    max_duty_cycle: float | None = None
    max_pulse_width: float | None = None
    external_trigger_pulse_repetitions: int | None = None
    out_width: float | None = None   # at present not used, but xc_testing requires it
    duty_cycle: float | None = None  # at present not used, but xc_testing requires it


@dataclass
class AdminMoveAlignMirrorDataset(GliderCommandDataset):
    align_index: int | None
    stage_angle: float | None


@dataclass
class AdminMoveAlignMirrorsDataset(GliderCommandDataset):
    stage1_angle: float | None
    stage2_angle: float | None
    stage3_angle: float | None
    stage4_angle: float | None


@dataclass
class AdminSmartAlignDataset(GliderCommandDataset):
    step: float | None
    mode: str


@dataclass
class AdminUpdateProfileDataset(GliderCommandDataset):
    cavity_index: int
    set_stage_angles: bool = None
    max_current: float | None = None
    max_voltage: float | None = None
    max_duty_cycle: float | None = None
    max_pulse_width: int | None = None
    applied_voltage: float | None = None
    current_limit: float | None = None
    pulse_period: int | None = None
    pulse_width: int | None = None
    temperature: float | None = None
    calibration: str = None
    preview: bool = False


@dataclass
class InitializeDataset(GliderCommandDataset):
    cavity_index : int


@dataclass
class AdminS2writeparametersDataset(GliderCommandDataset):
    cavity_index : int

def _check_adc_parameters(sampling_time_ns, oversampling, oversampling_shift,
                          channel_name):
    if sampling_time_ns not in ADC_SAMPLING_TIME_DICT_INV:
        raise ValueError('analog{}_sampling_time_ns shall be on of {}, '
                         'but {} was given'.format(channel_name,
                                                   ADC_SAMPLING_TIMES,
                                                   sampling_time_ns))
    if not 1 <= oversampling <= 1024:
        raise ValueError('analog{}_oversampling must be an integer between 1 and 1024, '
                         'but {} was given'.format(channel_name,
                                                   oversampling))
    if not 0 <= oversampling_shift <= 11:
        raise ValueError('analog{}_oversampling_shift must be an integer between 0 and 11, '
                         'but {} was given'.format(channel_name, oversampling_shift))


PGA_GAIN_1X = '1x'
PGA_GAIN_2X = '2x'
PGA_GAIN_4X = '4x'
PGA_GAIN_8X = '8x'
PGA_GAIN_16X = '16x'
MASK_REG_POI1_PGA_GAIN_ANALOG1 = 0x0007 # 0000 0000 0000 0111
MASK_REG_POI1_PGA_GAIN_ANALOG2 = 0x0038 # 0000 0000 0011 1000
PGA_GAIN = OrderedDict({PGA_GAIN_1X: 4,
                        PGA_GAIN_2X: 0,
                        PGA_GAIN_4X: 1,
                        PGA_GAIN_8X: 2,
                        PGA_GAIN_16X: 3,})


def is_valid_pga(pga: int):
    return pga in PGA_GAIN.values()


@dataclass
class OptimizeAdcDataset(GliderCommandDataset):
    wavenumber: float
    tuned_window_invcm: float
    stable_time_in_poi_ms: int
    adc_scan_size_us: int
    adc_step_size_ns: int
    adc_oversampling: int
    adc_oversampling_shift: int
    adc_sampling_time_ns: int
    analog_channel: int
    adc_start_time_ns: int = 0
    analog_pga: int = PGA_GAIN[PGA_GAIN_1X]

    def get_results_klass(self):
        return OptimizeAdcResults

    @model_validator(mode='after')
    def validate(self):
        if self.analog_channel not in [ANALOG_CHANNEL1, ANALOG_CHANNEL2]:
            raise ValueError('analog_channel must be one of {}, '
                             'but {} was give'.format([ANALOG_CHANNEL1, ANALOG_CHANNEL2],
                                                      self.analog_channel))
        _check_adc_parameters(sampling_time_ns=self.adc_sampling_time_ns,
                              oversampling=self.adc_oversampling,
                              oversampling_shift=self.adc_oversampling_shift,
                              channel_name='analog channel {}'.format(self.analog_channel))
        if not is_valid_pga(self.analog_pga):
            raise ValueError('analog_pga must be one of {}, but {} was given'.format(PGA_GAIN.values(),
                                                                                      self.analog_pga))


@dataclass
class SetProfileDataset(GliderCommandDataset):
    number: int


@dataclass
class ShutDownDataset(GliderCommandDataset):
    pass


STEPPING_POI_S2_TRIGGER_MODE_INT_BURST = 'internal_burst'
STEPPING_POI_S2_TRIGGER_MODE_INT_CONTINUOUS = 'internal_continuous'
STEPPING_POI_S2_TRIGGER_MODE_EXT = 'external'

STEPPING_POI_S2_TRIGGER_MODES = [STEPPING_POI_S2_TRIGGER_MODE_INT_BURST,
                                 STEPPING_POI_S2_TRIGGER_MODE_INT_CONTINUOUS,
                                 STEPPING_POI_S2_TRIGGER_MODE_EXT]


@dataclass
class SteppingPoiDataset(GliderCommandDataset):
    poi: List
    tuned_window_invcm: float
    stable_time_in_poi_ms: int
    use_analog1 : bool
    use_analog2 : bool
    analog1_delay_s2m_trigger_ns: int
    analog1_oversampling: int
    analog1_oversampling_shift: int
    analog1_sampling_time_ns: int
    analog2_delay_s2m_trigger_ns: int
    analog2_oversampling: int
    analog2_oversampling_shift: int
    analog2_sampling_time_ns: int
    s2_trigger_mode: str = STEPPING_POI_S2_TRIGGER_MODE_INT_BURST
    s2_external_trigger_timeout_ms: int | None = None
    include_analog_adc_buffers: bool = False
    repetitions: int = 1

    @classmethod
    def from_dict(cls, mydict):
        from glider.drivers.mcu import McuPOI
        rsp = cls(**mydict)
        rsp.poi = [McuPOI(**m) for m in rsp.poi]
        return rsp

    def get_results_klass(self):
        if not self.include_analog_adc_buffers:
            return SteppingPOIResults
        else:
            return SteppingPOIResultWithAnalogBuffers

    @model_validator(mode='after')
    def validate(self):
        if (not self.use_analog1) and (not self.use_analog2):
            raise ValueError('Either use_analog1 or use_analog2 shall be True, or both, '
                             'but both where set to False')
        if self.use_analog1:
            _check_adc_parameters(self.analog2_sampling_time_ns, self.analog2_oversampling,
                                  self.analog2_oversampling_shift, channel_name=1)
        if self.use_analog2:
            _check_adc_parameters(self.analog2_sampling_time_ns, self.analog2_oversampling,
                                  self.analog2_oversampling_shift, channel_name=2)


@dataclass
class SetWavenumberDataset(GliderCommandDataset):
    wavenumber: float
    pulsing_mode: str = 'internal'
    external_trigger_pulse_repetitions: int = 0
    calibrate_mcu_stage_position: bool = True


@dataclass
class MockCommandDataset(GliderCommandDataset):
    parameter: float | None = None
    raise_exception_while_executing: bool = False
    raise_exception_while_initializing: bool = False
    mcu_error_state: int = 0

    def get_results_klass(self):
        return MockCommandResults


@dataclass
class SwitchLasingOffDataset(GliderCommandDataset):
    pass


@dataclass
class ClearErrorsDataset(GliderCommandDataset):
    pass


@dataclass
class ClearEventsDataset(GliderCommandDataset):
    pass


RESULTS_COMMANDS = {'stepping_poi': SteppingPoiDataset,
                    'optimize_adc': OptimizeAdcDataset,
                    'mock_command': MockCommandDataset}






# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA" 

from pkg_resources import DistributionNotFound
from distutils.version import StrictVersion
from importlib.metadata import version as pkg_version

pkg = "glider_client"
try:
    version = pkg_version(pkg)
    try:
        StrictVersion(version)
    except ValueError as e:
        version = 'devel'
except DistributionNotFound:
    version = "devel"


from typing import List

import dataclasses
from dataclasses import dataclass


@dataclass
class GliderCommandResults:

    iter: int = None

    def _list_type_fields(self):
        return [f for f in dataclasses.fields(self) if f.name not in ['iter']]

    def extend(self, other):
        for f in self._list_type_fields():
            val = getattr(self, f.name)
            if val is None:
                val = []
                setattr(self, f.name, val)
            val.extend(getattr(other, f.name))

    def append(self, **kwargs):
        consumed = []
        for f in self._list_type_fields():
            if f.name not in kwargs:
                continue
            val = getattr(self, f.name)
            if val is None:
                val = []
                setattr(self, f.name, val)
            val.append(kwargs[f.name])
            consumed.append(f.name)
        if set(consumed) != set(kwargs.keys()):
            raise ValueError

@dataclass
class OptimizeAdcResults(GliderCommandResults):
    delayNs: List[int] = None
    position: List[int] = None
    adcSum: List[float] = None
    status: List[int] = None
    errors: List[str] = None
    cavity: List[int] = None


@dataclass
class SteppingPOIResults(GliderCommandResults):
    analog1AdcSum: List[float] = None
    analog2AdcSum: List[float] = None
    status: List[int] = None
    errors: List[str] = None
    wavenumber: List[float] = None
    acquiredPulsesNumber: List[int] = None
    cavity: List[int] = None
    laserDwellMs: List[int] = None
    postDwellMs: List[int] = None


@dataclass
class SteppingPOIResultWithAnalogBuffers(SteppingPOIResults):
    analog1BufferAddress: List[float] = None
    analog2BufferAddress: List[float] = None


@dataclass
class MockCommandResults(GliderCommandResults):
    myList: List[int] = None



# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import itertools
import os
from argparse import ArgumentParser
from datetime import datetime
from time import sleep
from copy import deepcopy

import requests

from glider_client.commands import SteppingPoiDataset, STEPPING_POI_S2_TRIGGER_MODE_INT_CONTINUOUS
from glider_client.glider import Glider
from glider_client.scripts.stepping_poi_plot import stepping_poi_plot
from glider_client.utils.alparser import _positive_float, _positive_int, _strict_positive_int
from glider_client.utils.mcu_registers import ADC_SAMPLING_TIMES
from glider_client.utils.ping import ping
from glider_client.utils.ssrv import store_stepping_poi


def run():

    glider_client = Glider(hostname='localhost',
                           port=5000)
    glider_status = glider_client.get_status()
    glider_client.initialize()

    wavenumbers = []

    for cp in glider_status.config.profiles[1].cavityProfiles.values():
        wavenumbers.append(min(cp.calibWnInvCm) + 1)
        wavenumbers.append(max(cp.calibWnInvCm) - 1)


    poi_list = [{'wavenumber': x,
                 'laserDwellMs': 0,
                 'postDwellMs': 0,
                 'numberOfPulses': 10,
                 'analog1PGA': 4,
                 'analog2PGA': 4,
                 } for x in sorted(wavenumbers)]

    use_analog1 = True
    use_analog2 = True

    parameters = {'poi': poi_list,
                  'tuned_window_invcm': 1,
                  'stable_time_in_poi_ms': 1,
                  'use_analog1': use_analog1,
                  'use_analog2': use_analog2,
                  'analog1_delay_s2m_trigger_ns': 200,
                  'analog1_oversampling': 3,
                  'analog1_oversampling_shift': 2,
                  'analog1_sampling_time_ns': 9,
                  'analog2_delay_s2m_trigger_ns': 200,
                  'analog2_oversampling': 3,
                  'analog2_oversampling_shift': 2,
                  'analog2_sampling_time_ns': 9,
                  's2_trigger_mode': STEPPING_POI_S2_TRIGGER_MODE_INT_CONTINUOUS,
                  }
    while True:
        command_proxy = glider_client.execute_command(SteppingPoiDataset(**parameters))

if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import os.path
from argparse import ArgumentParser

from matplotlib import pyplot
import numpy


def run():
    parser = ArgumentParser()
    parser.add_argument('files', nargs='+', type=str,
                        help='csv files containing the stepping POI')
    args = parser.parse_args()
    for fp in args.files:
        if not os.path.exists(fp):
            parser.error('file {} does not exist'.format(fp))
    scan_adc_delay_plot(args.files)


def scan_adc_delay_plot(files):
    for fp in files:
        n = numpy.loadtxt(fp, delimiter=',')
        pyplot.plot(n[:, 2], n[:, 0], label=os.path.basename(fp))
    pyplot.legend()
    pyplot.show()


if __name__ == '__main__':
    run()







# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import itertools
import os
from argparse import ArgumentParser
from datetime import datetime
from time import sleep
from copy import deepcopy

import requests

from glider_client.commands import SteppingPoiDataset
from glider_client.glider import Glider
from glider_client.scripts.stepping_poi_plot import stepping_poi_plot
from glider_client.utils.alparser import _positive_float, _positive_int, _strict_positive_int
from glider_client.utils.mcu_registers import ADC_SAMPLING_TIMES
from glider_client.utils.ping import ping
from glider_client.utils.ssrv import store_stepping_poi


def _analog_channel_number(value):
    value = int(value)
    if value not in [1, 2]:
        raise ValueError('Analog channel number must be 1 or 2')
    return value



def run():
    parser = ArgumentParser()
    parser.add_argument('-wn', nargs='+', type=_positive_float,
                        help='wavenumbers [cm-1]')
    parser.add_argument('-ld', nargs='+', type=_positive_int,
                        help='laser dwell time [ms]')
    parser.add_argument('-pd', nargs='+', type=_positive_int,
                        help='post dwell time [ms]')
    parser.add_argument('-c', type=str,
                        help='file path in CSV format, each line containing '
                             'wavenumber, laserDwellMs, postDwellMs')
    parser.add_argument('-wn_tol', type=_positive_float, required=True,
                        help='wavenumber tolerance window [cm-1]')
    parser.add_argument('-stab_poi', type=_positive_int, required=True,
                        help='stable time in POI [ms]')
    parser.add_argument('-pn', nargs='+', type=_strict_positive_int,
                        help='number of acquisition pulses')
    parser.add_argument('-ch', type=int, choices=[1, 2],
                        help='analog channel number, if not specified, both channels are used')
    args, remaining_args = parser.parse_known_args()

    analog_channels = [1, 2]

    if args.ch is not None:
        if args.ch == 1:
            analog_channels = [1]
        if args.ch == 2:
            analog_channels = [2]
    for c in [1, 2]:
        parser.add_argument('-anal{}_dl'.format(c), type=_positive_int, default=300,
                            help='analog{} acquisition delay after pulse trigger [ns]'.format(c))
        parser.add_argument('-anal{}_os'.format(c), type=int, default=3,
                            help='analog{} oversampling'.format(c))
        parser.add_argument('-anal{}_sh'.format(c), type=int, default=2,
                            help='analog{} oversampling shift'.format(c))
        parser.add_argument('-anal{}_sp'.format(c), type=_positive_int, default=9,
                            help='analog{} sampling time'.format(c), choices=ADC_SAMPLING_TIMES)
    parser.add_argument('-g', type=str, default='localhost', help='host name of glider' )
    if ping('ssrv'):
        parser.add_argument('--store_local', action='store_true', default=False,
                            help='force local storage of measurements')


    args = parser.parse_args()

    if ping('ssrv'):
        store_local = args.store_local
    else:
        store_local = True

    if args.c is None:
        if (args.wn is None) or (args.ld is None) or (args.pd is None) or (args.pn is None):
            parser.error('if you do not specify the -c option (CSV file path), '
                         'you will need to specify the POIs by the -wn, -ld, -pd and -pulse_num '
                         'options.')
        num_wavenumbers = len(args.wn)
        num_ld = len(args.ld)
        num_pd = len(args.pd)
        num_pulse_num = len(args.pn)
        wn_list = args.wn
        ld_list = []
        pd_list = []
        pulse_num_list = []
        if num_ld == 1:
            ld_list = num_wavenumbers*args.ld
        elif num_ld == num_wavenumbers:
            ld_list = args.ld
        else:
            parser.error('the length of -ld list must be either 1 or equal to the '
                         'length of the -wn list')
        if num_pd == 1:
            pd_list = num_wavenumbers*args.pd
        elif num_pd == num_wavenumbers:
            pd_list = args.pd
        else:
            parser.error('the length of -ld list must be either 1 or equal to the '
                         'length of the -wn list')
        if num_pulse_num == 1:
            pulse_num_list = num_wavenumbers*args.pn
        elif num_pulse_num == num_wavenumbers:
            pulse_num_list = args.pn
        else:
            parser.error('the length of -pn list must be either 1 or equal to the '
                         'length of the -wn list')
        poi_list = [{'wavenumber': a,
                     'laserDwellMs': b,
                     'postDwellMs': c,
                     'numberOfPulses': d} for a, b, c, d in itertools.zip_longest(wn_list,
                                                                                ld_list,
                                                                                pd_list,
                                                                                pulse_num_list)]
    else:
        if not os.path.exists(args.c):
            parser.error('csv file {} does not exists'.format(args.c))
        with open(args.c, 'r') as f:
            poi_list = [[float(x) for x in l.split(',')] for l in f.readlines()]
        poi_list = [{'wavenumber': x[0],
                     'laserDwellMs': int(x[1]),
                     'postDwellMs': int(x[2]),
                     'numberOfPulses': int(x[3])
                     } for x in poi_list]

    stepping_poi(glider_host=args.g, anal1_sp=args.anal1_sp, anal1_os=args.anal1_os, anal1_sh=args.anal1_sh,
                 anal2_sp=args.anal2_sp, anal2_os=args.anal2_os, anal2_sh=args.anal2_sh,
                 wn_tol=args.wn_tol, stab_poi=args.stab_poi,
                 channel=args.ch, anal1_dl=args.anal1_dl, anal2_dl=args.anal2_dl, poi_list=poi_list,
                 store_ssrv=(not store_local))


def stepping_poi(glider_host, anal1_sp, anal1_os, anal1_sh,
                 anal2_sp, anal2_os, anal2_sh, wn_tol, stab_poi,
                 channel, anal1_dl, anal2_dl, poi_list, store_ssrv=True,
                 plot=True):

    use_analog1 = True
    use_analog2 = True

    if channel is not None:
        if channel == 1:
            use_analog2 = False
        if channel == 2:
            use_analog1 = False

    results_dir = os.path.expanduser('~/glider/testing/stepping_poi/{}'.format(glider_host))

    if not os.path.exists(results_dir):
        os.makedirs(results_dir)

    results_file = os.path.join(results_dir, '{}_stepping_poi_'
                                             'st1_{}_os1_{}_sh1_{}'
                                             'st2_{}_os2_{}_sh2_{}.csv'.format(datetime.now().strftime('%Y_%m_%d_%H_%M_%S'),
                                                                               anal1_sp, anal1_os, anal1_sh,
                                                                            anal2_sp, anal2_os, anal2_sh))

    glider_client = Glider(hostname=glider_host,
                           port=5000)
    glider_client.initialize()

    parameters = {'poi': poi_list,
                  'tuned_window_invcm': wn_tol,
                  'stable_time_in_poi_ms': stab_poi,
                  'use_analog1': use_analog1,
                  'use_analog2': use_analog2,
                  'analog1_delay_s2m_trigger_ns': anal1_dl,
                  'analog1_oversampling': anal1_os,
                  'analog1_oversampling_shift': anal1_sh,
                  'analog1_sampling_time_ns': anal1_sp,
                  'analog2_delay_s2m_trigger_ns': anal2_dl,
                  'analog2_oversampling': anal2_os,
                  'analog2_oversampling_shift': anal2_sh,
                  'analog2_sampling_time_ns': anal2_sp,
                  }
    print(poi_list)
    print(parameters)

    command_proxy = glider_client.execute_command(SteppingPoiDataset(**parameters))

    # if False:
    if store_ssrv and ping('ssrv'):
        glider_config = glider_client.get_status().config
        dataset = deepcopy(parameters)
        dataset['wavenumber'] = command_proxy.result.wavenumber
        dataset['analog1adcsum_list'] = command_proxy.result.analog1AdcSum
        dataset['analog2adcsum_list'] = command_proxy.result.analog2AdcSum
        dataset['status_number'] = command_proxy.result.status
        dataset['cavity'] = command_proxy.result.cavity
        #dataset['errors'] = [x['errors'] for x in status['data_list']]
        dataset['acquiredpulsesnumber_list'] = command_proxy.result.acquiredPulsesNumber
        store_stepping_poi(glider_config, dataset)
    else:
        r = command_proxy.result
        with open(results_file, 'w') as f:
            for line in itertools.zip_longest(*[getattr(r, x)
                                                for x in ['wavenumber',
                                                          'analog1AdcSum',
                                                          'analog2AdcSum',
                                                          'acquiredPulsesNumber',
                                                          'cavity',
                                                          'errors',
                                                          'status']]):
                f.write(
                    '{}\n'.format(', '.join(str(x) for x in line)))
        if plot:
            stepping_poi_plot([results_file])


if __name__ == '__main__':
    run()







# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import itertools
import os
from argparse import ArgumentParser
from datetime import datetime

from glider_client.commands import OptimizeAdcDataset
from glider_client.glider import Glider
from glider_client.scripts.scan_adc_delay_plot import scan_adc_delay_plot
from glider_client.utils.alparser import _positive_int, _positive_float
from glider_client.utils.mcu_registers import ADC_SAMPLING_TIMES
from glider_client.utils.ping import ping
from glider_client.utils.ssrv import store_adc_delay


def run():
    parser = ArgumentParser()
    parser.add_argument('-wn', type=float, required=True, help='angle')
    parser.add_argument('-ch', type=int, required=False, choices=[1, 2],
                        default=2,
                        help='analog channel to be optimized')
    parser.add_argument('-scan', type=_positive_int, default=1,
                        help='scan size micro seconds')
    parser.add_argument('-step', type=_positive_int, default=4,
                        help='scan step in nano seconds (must be a multiple of 4)')
    parser.add_argument('-wn_tol', type=_positive_float, required=True,
                        help='wavenumber tolerance window [cm-1]')
    parser.add_argument('-samp_time', type=_positive_int, default=9,
                        help='adc sampling time [ns]', choices=ADC_SAMPLING_TIMES)
    parser.add_argument('-over_samp', type=int, default=3,
                        help='adc oversampling')
    parser.add_argument('-samp_shift', type=int, default=2,
                        help='adc sampling shift')
    parser.add_argument('-stab_poi', type=_positive_int, required=True,
                        help='stable time in POI [ms]')
    parser.add_argument('-g', type=str, default='localhost', help='host name of glider')
    if ping('ssrv'):
        parser.add_argument('--store_local', action='store_true', default=False,
                            help='force local storage of measurements')

    args = parser.parse_args()

    if ping('ssrv'):
        store_local = args.store_local
    else:
        store_local = True

    scan_adc_delay(glider_host=args.g, scan=args.scan, step=args.step, wn=args.wn,
                   samp_time=args.samp_time, over_samp=args.over_samp,
                   samp_shift=args.samp_shift, ch=args.ch, wn_tol=args.wn_tol,
                   stab_poi=args.stab_poi, store_ssrv=(not store_local))


def scan_adc_delay(glider_host, scan, step, wn,
                   samp_time, over_samp, samp_shift, ch, wn_tol,
                   stab_poi, store_ssrv=True, plot=True):
    glider_client = Glider(hostname=glider_host,
                           port=5000)
    glider_client.initialize()

    results_dir = os.path.expanduser('~/glider/testing/optimal_adc_delay/{}'.format(glider_host))

    if not os.path.exists(results_dir):
        os.makedirs(results_dir)

    results_file = os.path.join(results_dir,
                                '{}_optimal_delay_'
                                'scan_{}_step_{}_wn_{}_st_{}_os_{}_sh_{}.csv'.format(
                                    datetime.now().strftime('%Y_%m_%d_%H_%M_%S'),
                                    scan,
                                    step,
                                    wn,
                                    samp_time,
                                    over_samp,
                                    samp_shift))

    command_proxy = glider_client.execute_command(OptimizeAdcDataset(wavenumber=wn,
                                                                     tuned_window_invcm=wn_tol,
                                                                     analog_channel=ch,
                                                                     stable_time_in_poi_ms=stab_poi,
                                                                     adc_scan_size_us=scan,
                                                                     adc_step_size_ns=step,
                                                                     adc_oversampling=over_samp,
                                                                     adc_oversampling_shift=samp_shift,
                                                                     adc_sampling_time_ns=samp_time))

    if store_ssrv:
        glider_config = glider_client.get_status().config
        dataset = {'wavenumber': wn, 'analog_channel': ch, 'tuned_window_invcm': wn_tol,
                   'stable_time_in_poi_ms': stab_poi, 'adc_scan_size_us': scan, 'adc_step_size_ns': step,
                   'adc_oversampling': over_samp, 'adc_oversampling_shift': samp_shift,
                   'adc_sampling_time_ns': samp_time, 'adcsum_list': command_proxy.result.adcSum,
                   'delayns_list': command_proxy.result.delayNs, 'position': command_proxy.result.position,
                   'status_number': command_proxy.result.status, 'cavity': command_proxy.result.cavity[-1]}
        # dataset['error'] = status['data_list'][-1]['cavity']
        store_adc_delay(glider_config, dataset)
    else:
        r = command_proxy.result
        with open(results_file, 'w') as f:
            for line in itertools.zip_longest(*[getattr(r, x)
                                                for x in
                                                ['adcSum', 'cavity', 'delayNs',
                                                 'position', 'status']]):
                f.write(
                    '{}\n'.format(', '.join(str(x) for x in line)))
        if plot:
            scan_adc_delay_plot([results_file])


if __name__ == '__main__':
    run()



import platform    # For getting the operating system name
import subprocess  # For executing a shell command
import sys


def ping(host):
    """
    Returns True if host (str) responds to a ping request.
    Remember that a host may not respond to a ping (ICMP) request even if the host name is valid.
    """

    # Option for the number of packets as a function of
    param = '-n' if platform.system().lower()=='windows' else '-c'

    # Building the command. Ex: "ping -c 1 google.com"
    command = ['ping', param, '1', host]

    return subprocess.call(command, stdout=subprocess.DEVNULL) == 0


if __name__ == '__main__':
    print(ping('ssrv'))


# -*- coding: utf-8 -*-
"""
Created on September 01, 2015

Copyright Alpes Lasers SA, Neuchatel, Switzerland, 2015

@author: chiesa
"""

import argparse
import logging

logger = logging.getLogger(__name__)

_version = 'undefined'

try:
    from glider_client import pkg, version

    _version = '{} {}'.format(pkg, version)
except Exception as e:
    pkg = 'glider_client'
    logger.debug(e, exc_info=1)


def _positive_float(value):
    v = float(value)
    if v < 0.0:
        raise ValueError('{} must be a positive value'.format(v))
    return v

def _positive_int(value):
    v = int(value)
    if v < 0.0:
        raise ValueError('{} must be a positive value'.format(v))
    return v

def _strict_positive_int(value):
    v = int(value)
    if v <= 0.0:
        raise ValueError('{} must be a strict positive value'.format(v))
    return v


baseparser = argparse.ArgumentParser()
baseparser.add_argument("--show-version", help="Show the project version",
                        action="version", version="%s %s" % (pkg, version))







# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import dataclasses
import logging
from collections import OrderedDict
from dataclasses import dataclass
from typing import List, Any, Dict
from time import time, sleep

import numpy
import requests

from glider_client.commands import GliderCommandDataset, InitializeDataset, SwitchLasingOffDataset, ClearErrorsDataset, \
    RESULTS_COMMANDS

logger = logging.getLogger(__name__)


class GliderRuntimeError(Exception):
    pass


class GliderTimeout(Exception):
    pass


class GliderProtocolError(Exception):
    pass


class GliderProfileConfigurationError(Exception):
    pass


GLI_STATUS_CONNECTING = 'CONNECTING'  # initial state
GLI_STATUS_REQUIRE_FW_UPGRADE = 'REQUIRE FIRMWARE UPGRADE'  # some sub-components require
GLI_STATUS_NOT_INITIALIZED = 'NOT INITIALIZED'  # some motors not initialized or temperature
GLI_STATUS_INITIALIZING = 'INITIALIZING'  # initialization command running
GLI_STATUS_INITIALIZED = 'INITIALIZED'  # motors powered on, temperature stable, not lasing
GLI_STATUS_LASING = 'LASING'  # some laser is on and some temperature is stable
GLI_STATUS_EXECUTING_LASING = 'EXECUTING LASING'  # INITIALIZED and command running
GLI_STATUS_SYSTEM_ERROR = 'SYSTEM ERROR'  # any of the sub-systems is in error state
GLI_STATUS_SYSTEM_EXCURSION = 'SYSTEM EXCURSION'
GLI_STATUS_INTERLOCKED = 'INTERLOCKED'
GLI_STATUS_COMMUNICATION_ERROR = 'COMMUNICATION ERROR'
GLI_STATUS_ANY = 'ANY OPERATIONAL STATUS'

GLI_LASING_STATUS_NOT = 'NOT LASING'
GLI_LASING_STATUS_INIT = 'INIT'
GLI_LASING_STATUS_ARMED = 'ARMED'


def get_glider_level_from_logging_level(logging_level):
    if logging_level <= logging.INFO:
        return LEVEL_INFO
    elif logging_level <= logging.WARNING:
        return LEVEL_WARNING
    else:
        return LEVEL_ERROR


LEVEL_SUCCESS = 'success'
LEVEL_INFO = 'info'
LEVEL_WARNING = 'warning'
LEVEL_ERROR = 'error'
LEVEL_SEVERITY = [LEVEL_ERROR, LEVEL_WARNING, LEVEL_INFO, LEVEL_SUCCESS]


class BitRegister:

    def __init__(self, value):
        self._value = value

    def get_value(self):
        return self._value

    def _get_bit(self, position):
        return bool(self._value & (1 << position))

    def get_user_msg(self):
        raise NotImplementedError

    def to_dict(self):
        rsp = OrderedDict()
        for a in dir(self):
            if a.startswith('_'):
                continue
            if a not in ['update', 'to_dict', 'get_value',
                         'get_user_msg', 'reset_parameters']:
                rsp[a] = getattr(self, a)
        return rsp

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()

    def __repr__(self):
        return '<{}({}:{})>'.format(self.__class__.__name__,
                                    self._value,
                                    ', '.join(['{}={}'.format(k, v)
                                               for k, v in self.to_dict().items()]))


class SettableBitRegister(BitRegister):

    def _set_bit(self, position, value):
        if value:
            self._set_bit_true(position)
        else:
            self._set_bit_false(position)

    def _set_bit_true(self, position):
        self._value |= (1<<position)

    def _set_bit_false(self, position):
        self._value &= ~(1<<position)


def set_bit_register(br):
    return br.get_value()


class CmdParams1(SettableBitRegister):

    def reset_parameters(self):
        """Reset all, except for global parameters (as for instance debug_mode"""
        debug_mode = self.debugMode
        self._value = 0
        self.debugMode = debug_mode

    @property
    def debugMode(self):
        return self._get_bit(0)

    @debugMode.setter
    def debugMode(self, value):
        self._set_bit(0, value)

    @property
    def OUT9AsAnalog1AdcTrigger(self):
        """if this bit is set: OUT_9 on subD connector will output Analog 1 ADC Trigger, else, Process Output signal"""
        return self._get_bit(1)

    @OUT9AsAnalog1AdcTrigger.setter
    def OUT9AsAnalog1AdcTrigger(self, value):
        self._set_bit(1, value)

    @property
    def OUT8AsPositionRange(self):
        """if this bit is set: OUT_8 on subD connector will output DAC boundary signal, else, Tuned Output signal"""
        return self._get_bit(2)

    @OUT8AsPositionRange.setter
    def OUT8AsPositionRange(self, value):
        self._set_bit(2, value)

    @property
    def stageReversedCounting(self):
        """if this bit is set: reverse TIM2 encoder direction by setting sConfig.IC1Polarity = TIM_ICPOLARITY_FALLING;"""
        return self._get_bit(3)

    @stageReversedCounting.setter
    def stageReversedCounting(self, value):
        self._set_bit(3, value)


class S2StatusRegister(BitRegister):

    @property
    def hasErrors(self):
        return self._value != 0

    @property
    def isUnderVoltage(self):
        return self._get_bit(0)

    @property
    def isOverCurrent(self):
        return self._get_bit(1)

    @property
    def isOverVoltage(self):
        return self._get_bit(2)

    @property
    def isOverTemp(self):
        return self._get_bit(3)

    @property
    def isFastOverCurrent(self):
        return self._get_bit(4)

    @property
    def isOutOfPulseOverCurrent(self):
        return self._get_bit(5)

    @property
    def bootFailed(self):
        return self._get_bit(6)

    def to_user_message(self):
        if not self.hasErrors:
            return '{}: OK'.format(self.__class__.__name__)
        return '; '.join([k.replace('is', '') for k, v in self.to_dict().items()
                          if k not in ['hasErrors'] and v is True])


class ECErrorRegister(BitRegister):

    @property
    def hasErrors(self):
        return self._value != 0

    @property
    def initFail(self):
        return self._get_bit(0)

    @property
    def timeoutMotor(self):
        return self._get_bit(1)

    @property
    def positionOutOfBonds(self):
        return self._get_bit(2)

    @property
    def invalidParameters(self):
        return self._get_bit(3)

    @property
    def stopRequest(self):
        return self._get_bit(4)

    @property
    def pulseTimeout(self):
        return self._get_bit(5)

    @property
    def analog2AdcTimeout(self):  # analog 2 >>> adc1
        return self._get_bit(6)

    @property
    def analog1AdcTimeout(self):  # analog 1 >>> adc2
        return self._get_bit(7)

    @property
    def dmaTimeout(self):
        return self._get_bit(8)

    @property
    def uart1Timeout(self):
        return self._get_bit(9)

    @property
    def msmInvalidState(self):
        return self._get_bit(10)

    @property
    def registerOverflow(self):
        return self._get_bit(11)

    @property
    def analog1Clipping(self):
        return self._get_bit(12)

    @property
    def analog2Clipping(self):
        return self._get_bit(13)

    @property
    def outOfMemory(self):
        return self._get_bit(14)

    def get_user_msg(self):
        return ', '.join([str(error_name) for error_name, error_is_set in self.to_dict().items()
                          if error_is_set]) or 'ok'


class ECInterlocksRegister(BitRegister):

    @property
    def userInterlockActive(self):
        return self._get_bit(0)

    @property
    def physicalInterlockActive(self):
        return self._get_bit(1)

    @property
    def tecInterlockActive(self):
        return self._get_bit(2)


class OutsideCalibrationBounds(Exception):
    pass


class WavenumberOutsideCalibrationBounds(OutsideCalibrationBounds):
    pass


class AngleOutsideCalibrationBounds(OutsideCalibrationBounds):
    pass


MONOTONIC_INCREASING = 'increasing'
MONOTONIC_DECREASING = 'decreasing'
NOT_MONOTONIC = 'not monotonic'


def analyse_monotonicity(alist):
    if len(alist) <= 1:
        raise ValueError('Cannot determine monotonicity of a one element list')
    sign = 1
    if alist[1] < alist[0]:
        sign = -1
    if numpy.all(sign * numpy.diff(numpy.array(alist)) > 0):
        return MONOTONIC_INCREASING if sign > 0 else MONOTONIC_DECREASING
    return NOT_MONOTONIC


def interpolate(x, xp, fp):
    if xp[1] < xp[0]:
        return numpy.interp(x, numpy.flip(xp), numpy.flip(fp))
    return numpy.interp(x, xp, fp)


def get_calibration_errors(angle_deg, wavenumber_invcm, power_mW):
    errors = []
    try:
        if type(angle_deg) != list:
            errors.append('"angle_deg" shall be a list')
        if type(wavenumber_invcm) != list:
            errors.append('"wavenumber_invcm" shall be a list')
        if type(power_mW) != list:
            errors.append('"power_mW" shall be a list')
        if errors:
            return errors
        if not (len(angle_deg) == len(wavenumber_invcm) == len(power_mW)):
            errors.append('angle_deg, wavenumber_invcm and power_mW '
                          'columns must have the same size')
            return errors
        if len(angle_deg) <= 1:
            errors.append('calibration list has length {}, '
                          'expected at least 2 elements'.format(len(angle_deg)))
            return errors
        if analyse_monotonicity(angle_deg) == NOT_MONOTONIC:
            errors.append('calibration angles are not monotonic')
        if analyse_monotonicity(wavenumber_invcm) == NOT_MONOTONIC:
            errors.append('wavenumber are not monotonic')
        for c in power_mW:
            try:
                float(c)  # verify it's a float
            except ValueError:
                errors.append('power_mW shall be a list of float, got {}'.format(c)
                              )
                return errors
    except Exception as e:
        logger.exception(e, exc_info=1)
        errors.append(str(e))
    return errors


@dataclass
class GliderCavityProfile:
    index: int
    appliedVoltage: float
    currentLimit: float
    pulsePeriod: float
    pulseWidth: float
    temperature: float
    stage1Angle: float
    stage2Angle: float
    stage3Angle: float
    stage4Angle: float
    maxVoltage: float
    maxDutyCycle: float
    maxCurrent: float
    maxPulseWidth: float
    calibAnglesDeg: list = None
    calibWnInvCm: list = None
    calibPowerMW: list = None

    @property
    def pulsePeriodMCUExternalTriggerFactor(self):
        return 0.95

    def get_errors(self):
        errors = []
        if (self.calibAnglesDeg is not None or
                self.calibWnInvCm is not None or
                self.calibPowerMW is not None):
            errors.extend(get_calibration_errors(angle_deg=self.calibAnglesDeg,
                                                 wavenumber_invcm=self.calibWnInvCm,
                                                 power_mW=self.calibPowerMW))
        return errors

    def get_stage_angle(self, index):
        return getattr(self, 'stage{}Angle'.format(index))

    def is_wavenumber_within_bounds(self, wavenumber):
        return min(self.calibWnInvCm) <= wavenumber <= max(self.calibWnInvCm)

    def is_angle_within_bounds(self, angle):
        return min(self.calibAnglesDeg) <= angle <= max(self.calibAnglesDeg)

    def get_angle_from_wavenumber(self, wavenumber):
        if not self.is_wavenumber_within_bounds(wavenumber):
            raise WavenumberOutsideCalibrationBounds('wavenumber {} outside of '
                                                     'calibration bounds for cavity {}'.format(wavenumber,
                                                                                               self.index))
        return interpolate(numpy.array(wavenumber),
                           numpy.array(self.calibWnInvCm),
                           numpy.array(self.calibAnglesDeg))

    def get_power_from_wavenumber(self, wavenumber):
        if not self.is_wavenumber_within_bounds(wavenumber):
            raise WavenumberOutsideCalibrationBounds('wavenumber {} outside of '
                                                     'calibration bounds for cavity {}'.format(wavenumber,
                                                                                               self.index))
        return interpolate(numpy.array(wavenumber),
                           numpy.array(self.calibWnInvCm),
                           numpy.array(self.calibPowerMW))

    def get_wavenumber_from_angle(self, angle):
        if not self.is_angle_within_bounds(angle):
            raise AngleOutsideCalibrationBounds
        return interpolate(numpy.array(angle),
                           numpy.array(self.calibAnglesDeg),
                           numpy.array(self.calibWnInvCm))

    def get_angle_tolerance_from_wavenumber_tolerance(self,
                                                      wavenumber_tolerance,
                                                      wavenumber_list):
        wavenumber_list = [w for w in wavenumber_list if
                           self.is_wavenumber_within_bounds(w + wavenumber_tolerance / 2.0)
                           and self.is_wavenumber_within_bounds(w - wavenumber_tolerance / 2.0)]
        if not wavenumber_list:
            raise ValueError('Cannot determine motor tolerance from '
                             'the provided wavenumbers and wavenumber tolerance: '
                             'all of them are out of calibration bounds')
        angles_minus = interpolate(numpy.array(wavenumber_list) - wavenumber_tolerance / 2.0,
                                   numpy.array(self.calibWnInvCm),
                                   numpy.array(self.calibAnglesDeg))
        angles_plus = interpolate(numpy.array(wavenumber_list) + wavenumber_tolerance / 2.0,
                                  numpy.array(self.calibWnInvCm),
                                  numpy.array(self.calibAnglesDeg))
        return numpy.max(numpy.abs(angles_plus - angles_minus))


@dataclass
class GliderProfile:
    number: int
    cavityProfiles: Dict[int, GliderCavityProfile] = None

    def get_cavity_and_angle_for_wavenumber(self, wavenumber):
        """Returns best cavity for a given wavenumber, where the best cavity
        is the one which gives the higher power
        """
        candidates = [c for c in self.cavityProfiles.values()
                      if c.is_wavenumber_within_bounds(wavenumber)]
        if not candidates:
            raise WavenumberOutsideCalibrationBounds('wavenumber {} outside '
                                                     'of calibration bounds'.format(wavenumber))
        c = sorted(candidates,
                   key=lambda x: x.get_power_from_wavenumber(wavenumber))[-1]
        return c, c.get_angle_from_wavenumber(wavenumber)

    def get_cavity_profile(self, index):
        if index not in self.cavityProfiles:
            raise ValueError('profile for cavity {} not defined'.format(index))
        return self.cavityProfiles[index]


@dataclass
class BoardConfig:
    usbSerial: str
    devPathTemplate: str
    checkVersions: bool

    def _get_device_path(self, interface):
        return self.devPathTemplate.format(self.usbSerial, interface)


@dataclass
class CavityConfig(BoardConfig):
    index: int
    stageAxis: int
    laserSerial: str
    stageReversedCounting: bool
    pgaDeltaDelay2xNs: int
    pgaDeltaDelay4xNs: int
    pgaDeltaDelay8xNs: int
    pgaDeltaDelay16xNs: int

    @property
    def stagePath(self):
        return self._get_device_path('00')

    @property
    def s2Path(self):
        return self._get_device_path('01')

    @property
    def mcuPath(self):
        return self._get_device_path('02')

    @property
    def tecPath(self):
        return self._get_device_path('03')


@dataclass
class AlignConfig(BoardConfig):

    stage1ReversedCounting: bool
    stage2ReversedCounting: bool
    stage3ReversedCounting: bool
    stage4ReversedCounting: bool

    def get_stage_path(self, index):
        return self._get_device_path('0{}'.format(index - 1))

    def get_stage_reversed_counting(self, index):
        return getattr(self, 'stage{}ReversedCounting'.format(index))


@dataclass
class GliderConfig:
    cavitiesNumber: int = None
    serial: int = None
    cavities: Dict[int, CavityConfig] = None
    align: AlignConfig = None
    profiles: Dict[int, GliderProfile] = None
    version: str = None
    forcePhysicalInterlockActive: bool = False

    def get_cavity_config(self, index) -> CavityConfig:
        return self.cavities[index]

    def get_profile(self, number=None):
        if not self.profiles:
            raise GliderProfileConfigurationError('No glider profiles are configured.')
        if number is None:
            if len(self.profiles) != 1:
                raise GliderProfileConfigurationError('More than one profile configured, '
                                 'please specify profile number')
            return list(self.profiles.values())[0]
        return self.profiles[number]


@dataclass
class UserMessage:
    message: str = None
    level: str = None


@dataclass
class DeviceStatus:
    driverPath: str = None
    driverPathExists: bool = None
    lastCommunicationError: str = None
    requiresUpgrade: bool = None
    isRunning: bool = None
    connectionOK: bool = None
    hasCommunicationErrors: bool = None
    userMessage: UserMessage = None

    @property
    def hasErrors(self):
        return False


@dataclass
class S2Status(DeviceStatus):
    measuredVoltage: float = None
    measuredCurrent: float = None
    pulsePeriod: float = None
    pulseWidth: float = None
    dutyCycle: float = None
    appliedVoltage: float = None
    currentLimit: float = None
    pulsingMode: float = None
    statusRegister: S2StatusRegister = None
    maxVoltage: float = None
    maxDutyCycle: float = None
    maxCurrent: float = None
    maxPulseWidth: float = None
    externalTriggerPulseRepetitions: int = None

    @property
    def isLasing(self):
        return self.pulsingMode != 'off'

    @property
    def hasErrors(self):
        if self.statusRegister:
            return self.statusRegister.hasErrors
        return False


@dataclass
class StageStatus(DeviceStatus):
    driverSerialNumber: str = None
    actualAngle: float = None
    commandAngle: float = None
    motionErrorRegister: Any = None
    statusRegisterLow: Any = None
    statusRegisterHigh: Any = None

    @property
    def isInitialized(self):
        if self.statusRegisterLow:
            if self.statusRegisterLow.isAxisOn is True:
                return True
        return False


    @property
    def hasErrors(self):
        if self.motionErrorRegister:
            return self.motionErrorRegister.hasErrors
        return False

@dataclass
class MCUStatus(DeviceStatus):
    status: str = None
    command: str = None
    mode: str = None
    numberPOI: int = None
    numberAcquisitionsPerPOI: int = None
    POITolerance: int = None
    ecError: ECErrorRegister = None
    machineStatus: str = None
    isActiveEc: bool = None
    currentPOI: int = None
    encoderPosition: int = None
    encoderAngle: float = None
    interlockStatus: ECInterlocksRegister = None
    externalModeTimeout: int = None
    temperature: float = None
    humidity: float = None
    encoderDACMin: int = None
    encoderDACMax: int = None
    cmdParams1: CmdParams1 = None

    @property
    def hasErrors(self):
        if self.ecError:
            return self.ecError.hasErrors

    @property
    def allInterlocksActive(self):
        return (self.interlockStatus.userInterlockActive and
                self.interlockStatus.physicalInterlockActive)


@dataclass
class TECStatus(DeviceStatus):
    measuredTemperature: float = None
    sinkTemperature: float = None
    targetTemperature: float = None
    tecVoltage: float = None
    tecCurrent: float = None
    isStable: bool = None
    isOutputEnabled: bool = None
    status: int = None
    statusLabel: str = None
    hasError: bool = None
    errorNumber: int = None

    def is_temperature_stable(self, temperature):
        if ((not self.hasErrors)
                and self.isOutputEnabled
                and self.isStable):
            if self.targetTemperature == temperature:
                return True
        return False

    @property
    def hasErrors(self):
        return self.status == 3  # almetec.parameters.STATUS_ERROR


@dataclass
class AlignStatus:
    stage1Status: StageStatus = None
    stage2Status: StageStatus = None
    stage3Status: StageStatus = None
    stage4Status: StageStatus = None

    @property
    def hasSubStates(self):
        return self.stage1Status and self.stage2Status and self.stage3Status and self.stage4Status

    @property
    def connectionOK(self):
        if self.hasSubStates:
            return (self.stage1Status.connectionOK and
                    self.stage2Status.connectionOK and
                    self.stage3Status.connectionOK and
                    self.stage4Status.connectionOK)

    @property
    def requiresUpgrade(self):
        if self.hasSubStates:
            return (self.stage1Status.requiresUpgrade or
                    self.stage2Status.requiresUpgrade or
                    self.stage3Status.requiresUpgrade or
                    self.stage4Status.requiresUpgrade)

    @property
    def hasErrors(self):
        if self.hasSubStates:
            return (self.stage1Status.hasErrors or
                    self.stage2Status.hasErrors or
                    self.stage3Status.hasErrors or
                    self.stage4Status.hasErrors)

    @property
    def hasCommunicationErrors(self):
        if self.hasSubStates:
            return (self.stage1Status.hasCommunicationErrors or
                    self.stage2Status.hasCommunicationErrors or
                    self.stage3Status.hasCommunicationErrors or
                    self.stage4Status.hasCommunicationErrors)

    @property
    def isInitialized(self):
        if self.hasSubStates:
            return (self.stage1Status.isInitialized and
                    self.stage2Status.isInitialized and
                    self.stage3Status.isInitialized and
                    self.stage4Status.isInitialized)


@dataclass
class CavityStatus:
    s2: S2Status = None
    stage: StageStatus = None
    mcu: MCUStatus = None
    tec: TECStatus = None

    @property
    def isTECOn(self):
        if self.tec:
            return self.tec.isOutputEnabled
        return False

    @property
    def isStageInitialized(self):
        if self.stage:
            return self.stage.isInitialized
        return False

    @property
    def allInterlocksActive(self):
        if self.mcu:
            return self.mcu.allInterlocksActive
        return False

    @property
    def connectionOK(self):
        if self.hasSubStates:
            return (self.s2.connectionOK and
                    self.stage.connectionOK and
                    self.mcu.connectionOK and
                    self.tec.connectionOK)

    @property
    def requiresUpgrade(self):
        if self.hasSubStates:
            return (self.s2.requiresUpgrade or
                    self.stage.requiresUpgrade or
                    self.mcu.requiresUpgrade or
                    self.tec.requiresUpgrade)

    @property
    def hasErrors(self):
        if self.hasSubStates:
            return (self.s2.hasErrors or
                    self.stage.hasErrors or
                    self.mcu.hasErrors or
                    self.tec.hasErrors)

    @property
    def hasCommunicationErrors(self):
        if self.hasSubStates:
            return (self.s2.hasCommunicationErrors or
                    self.stage.hasCommunicationErrors or
                    self.mcu.hasCommunicationErrors or
                    self.tec.hasCommunicationErrors)

    @property
    def hasSubStates(self):
        return self.s2 and self.stage and self.mcu and self.tec

    @property
    def isLasing(self):
        if self.s2:
            return self.s2.isLasing

    @property
    def isTecOn(self):
        if self.tec:
            return self.tec.isOutputEnabled


@dataclass
class GliderCommandMessage:
    level: str
    message: str


@dataclass
class GliderCommandStatus:
    level: str = None
    messages: List[GliderCommandMessage] = None
    exception: Any = None
    name: str = None
    response: Any = None
    id: str = None
    parameters: None = None
    isExecuting: bool = False
    hasExecuted: bool = False
    result: None = None
    results: List = None
    results_num: int = None
    progress: int = 0

    @classmethod
    def from_dict(cls, mydict, DataSetKlass):
        rsp = cls()
        rsp.level = mydict['level']
        rsp.exception = mydict['exception']
        rsp.name = mydict['name']
        rsp.response = mydict['response']
        rsp.id = mydict['id']
        rsp.parameters = DataSetKlass(**mydict['parameters'])
        rsp.isExecuting = mydict['isExecuting']
        rsp.hasExecuted = mydict['hasExecuted']
        rsp.result = rsp.parameters.get_results_klass()(**mydict['result'])
        rsp.results_num = mydict['results_num']
        rsp.progress = mydict['progress']
        rsp.messages=[GliderCommandMessage(**m) for m in mydict['messages']]
        return rsp

    def add_message(self, message, level=logging.INFO):
        glider_level = get_glider_level_from_logging_level(level)
        self.messages.append(GliderCommandMessage(level=level,
                                                  message=message))
        if not self.level:
            self.level = glider_level
        else:
            if LEVEL_SEVERITY.index(glider_level) < LEVEL_SEVERITY.index(self.level):
                self.level = glider_level


@dataclass
class SystemExcursionEvent:
    time: float
    msg: str



SYSTEM_EXCURSION_EVENT_INTERLOCK_USER = 'user interlock'
SYSTEM_EXCURSION_EVENT_INTERLOCK_PHYSICAL = 'physical interlock'
SYSTEM_EXCURSION_EVENT_INTERLOCK_TEC = 'tec interlock'


SYSTEM_EXCURSION_EVENTS = [SYSTEM_EXCURSION_EVENT_INTERLOCK_PHYSICAL,
                           SYSTEM_EXCURSION_EVENT_INTERLOCK_USER,
                           SYSTEM_EXCURSION_EVENT_INTERLOCK_TEC]



@dataclass
class GliderStatus:
    cavities: Dict[int, CavityStatus] = None
    align: AlignStatus = None
    config: GliderConfig = None
    isCommandRunning: bool = None
    isRunning: bool = None
    isConfigured: bool = None
    selectedProfile: int = None
    outputWavenumber: float = None
    hasSystemExcursionEvents: bool = False
    _operationalStatus: str = None

    @classmethod
    def from_dict(cls, mydict):
        operational_status = mydict['operationalStatus'] if 'operationalStatus' in mydict else mydict['_operationalStatus']
        glider_status = cls(cavities={},
                  isCommandRunning=mydict['isCommandRunning'],
                  isRunning=mydict['isRunning'],
                  isConfigured=mydict['isConfigured'],
                  selectedProfile=mydict['selectedProfile'],
                  outputWavenumber=mydict['outputWavenumber'],
                  _operationalStatus=operational_status)
        for k, v in mydict['cavities'].items():
            cavity_status = CavityStatus()
            glider_status.cavities[int(k)] = cavity_status
            cavity_status.s2 = S2Status(**v['s2'])
            cavity_status.stage = StageStatus(**v['stage'])
            cavity_status.mcu = MCUStatus(**v['mcu'])
            cavity_status.tec = TECStatus(**v['tec'])
        if mydict['align']:
            glider_status.align = AlignStatus(stage1Status=StageStatus(**mydict['align']['stage1Status']),
                                           stage2Status=StageStatus(**mydict['align']['stage2Status']),
                                           stage3Status=StageStatus(**mydict['align']['stage3Status']),
                                           stage4Status=StageStatus(**mydict['align']['stage4Status']))
        glider_status.config = GliderConfig(cavitiesNumber=mydict['config']['cavitiesNumber'],
                                         serial=mydict['config']['serial'],
                                         version=mydict['config']['version'],
                                         cavities={},
                                         profiles={})
        if mydict['config']['align']:
            glider_status.config.align = AlignConfig(**mydict['config']['align'])
        for k, v in mydict['config']['cavities'].items():
            glider_status.config.cavities[int(k)] = CavityConfig(**v)
        for profile_number, profile_values in mydict['config']['profiles'].items():
            profile = GliderProfile(number=profile_values['number'],
                                    cavityProfiles={})
            for cavity_index, cavity_profile_values in profile_values['cavityProfiles'].items():
                profile.cavityProfiles[int(cavity_index)] = GliderCavityProfile(**cavity_profile_values)
            glider_status.config.profiles[int(profile_number)] = profile
        return glider_status

    @property
    def connectionOK(self):
        if self.align and not self.align.connectionOK:
            return False
        for c in self.cavities.values():
            if not c.connectionOK:
                return False
        return True

    def get_profile(self):
        return self.config.get_profile(number=self.selectedProfile)

    @property
    def isAnyTECOn(self):
        for c in self.cavities.values():
            if c.isTECOn:
                return True
        return False

    @property
    def isTemperatureStable(self):
        profile = self.get_profile()
        for cavity_index, cavity in self.cavities.items():
            cavity_profile = profile.get_cavity_profile(cavity_index)
            if not cavity.tec.is_temperature_stable(cavity_profile.temperature):
                return False
        return True

    @property
    def requiresUpgrade(self):
        if self.align and self.align.requiresUpgrade:
            return True
        for c in self.cavities.values():
            if c.requiresUpgrade:
                return True
        return False

    @property
    def hasSubStates(self):
        if self.align and not self.align.hasSubStates:
            return False
        for c in self.cavities.values():
            if not c.hasSubStates:
                return False
        return True

    @property
    def hasErrors(self):
        if self.align and self.align.hasErrors:
            return True
        for c in self.cavities.values():
            if c.hasErrors:
                return True
        return False

    @property
    def hasCommunicationErrors(self):
        if self.align and self.align.hasCommunicationErrors:
            return True
        for c in self.cavities.values():
            if c.hasCommunicationErrors:
                return True
        return False

    @property
    def allInterlocksActive(self):
        for c in self.cavities.values():
            if not c.allInterlocksActive:
                return False
        return True

    @property
    def isShutDown(self):
        if self.align and self.align.isInitialized:
            return False
        for c in self.cavities.values():
            if c.isStageInitialized:
                return False
            if c.isLasing:
                return False
            if c.isTecOn:
                return False
        return True

    @property
    def isInitialized(self):
        if self.align and not self.align.isInitialized:
            return False
        for c in self.cavities.values():
            if not c.isStageInitialized:
                return False
        if not self.isTemperatureStable:
            return False
        return True

    @property
    def isLasing(self):
        for c in self.cavities.values():
            if c.isLasing:
                return True
        return False

    @property
    def lasingStatus(self):
        if self.operationalStatus == GLI_STATUS_INITIALIZING:
            return GLI_LASING_STATUS_INIT
        if self.isAnyTECOn:
            return GLI_LASING_STATUS_ARMED
        return GLI_LASING_STATUS_NOT

    @property
    def operationalStatus(self):
        if self._operationalStatus:
            return self._operationalStatus
        if not self.hasSubStates:
            return GLI_STATUS_CONNECTING
        if self.connectionOK:
            if self.requiresUpgrade:
                return GLI_STATUS_REQUIRE_FW_UPGRADE
            if self.hasErrors:
                return GLI_STATUS_SYSTEM_ERROR
            if self.hasSystemExcursionEvents:
                return GLI_STATUS_SYSTEM_EXCURSION
            if not self.allInterlocksActive:
                return GLI_STATUS_INTERLOCKED
            if self.isInitialized:
                if self.isLasing:
                    if self.isCommandRunning:
                        return GLI_STATUS_EXECUTING_LASING
                    return GLI_STATUS_LASING
                return GLI_STATUS_INITIALIZED
            else:
                if self.isCommandRunning:
                    return GLI_STATUS_INITIALIZING
                return GLI_STATUS_NOT_INITIALIZED
        else:
            if self.hasCommunicationErrors:
                return GLI_STATUS_COMMUNICATION_ERROR
            return GLI_STATUS_CONNECTING


class GliderCommandProxy:

    status = GliderCommandStatus

    def __init__(self, glider, command_id, dataset):
        self.id = command_id
        self.status = None
        self.glider = glider
        self.dataset = dataset

    def update(self):
        rsp = requests.get('{}/api/command'.format(self.glider.url))
        rsp.raise_for_status()
        status = rsp.json()
        if status['id'] != self.id:
            raise Exception('the current running command ID does not '
                            'match the local ID: {} != {}'.format(status['id'],
                                                                  self.id))
        self.status = GliderCommandStatus.from_dict(status,
                                                    DataSetKlass=self.dataset.__class__)

    def _check_status_loaded(self):
        if not self.status:
            raise Exception('status object not loaded, call update()')

    @property
    def hasErrors(self):
        self._check_status_loaded()
        return self.status.level == 'error'

    @property
    def errorMessage(self):
        self._check_status_loaded()
        return self.status.exception

    def stop(self):
        rsp = requests.post('{}/api/command/stop'.format(self.glider.url))
        rsp.raise_for_status()

    @property
    def isExecuting(self):
        self._check_status_loaded()
        return self.status.isExecuting

    @property
    def hasExecuted(self):
        self._check_status_loaded()
        return self.status.hasExecuted

    @property
    def result(self):
        self._check_status_loaded()
        return self.status.result


class Glider:

    def __init__(self, hostname='localhost', port=5000,
                 is_mock=False):
        self._hostname = hostname
        self._port = port
        self.isMock = is_mock
        self.testPhysicalInterlock = None
        self.testUserInterlock = None
        self.testController = None

    @property
    def url(self):
        return 'http://{}:{}'.format(self._hostname, self._port)

    def get_status(self) -> GliderStatus:
        rsp = requests.get('{}/api/status'.format(self.url))
        rsp.raise_for_status()
        return GliderStatus.from_dict(mydict=rsp.json())

    def initialize(self, active_cavity_index=1, timeout=3*60):
        start_time = time()
        status = self.get_status()
        if status.operationalStatus == GLI_STATUS_SYSTEM_ERROR:
            self.clear_errors()
        if status.operationalStatus in [GLI_STATUS_INITIALIZED, GLI_STATUS_LASING, GLI_STATUS_EXECUTING_LASING]:
            return
        self.execute_command(command_dataset=
                                   InitializeDataset(
                                       cavity_index=active_cavity_index),
                                 timeout=timeout)

        while True:
            status = self.get_status()
            if status.operationalStatus in [GLI_STATUS_INITIALIZED, GLI_STATUS_LASING, GLI_STATUS_EXECUTING_LASING]:
                return
            if time() - start_time > timeout:
                raise GliderTimeout('current status is {}'.format(status.operationalStatus))
            sleep(0.1)

    def wait_idle(self, timeout=2):
        start_time = time()
        while time() - start_time < timeout:
            current_status = self.get_status()
            if not current_status.isCommandRunning:
                return
        raise GliderTimeout

    def wait_status(self, target_op_status, timeout=2):
        start_time = time()
        while time() - start_time < timeout:
            current_status = self.get_status()
            if current_status.operationalStatus == target_op_status:
                return
        current_status = self.get_status()
        current_op_status = current_status.operationalStatus
        if current_op_status != target_op_status:
            raise GliderTimeout('glider current state is {}, but {} was expected'.format(current_op_status,
                                                                                        target_op_status))

    def switch_lasing_off(self):
        self.execute_command(command_dataset=SwitchLasingOffDataset())

    def clear_errors(self):
        self.execute_command(command_dataset=ClearErrorsDataset())

    def execute_command(self, command_dataset: GliderCommandDataset, timeout=3*60) -> GliderCommandProxy:
        command_proxy = None
        try:
            command_proxy = self.execute_command_async(command_dataset=command_dataset)
            start_time = time()
            while True:
                if time() - start_time > timeout:
                    raise GliderTimeout
                command_proxy.update()
                if command_proxy.hasExecuted:
                    break
                sleep(0.1)
            command_proxy.update()
            if command_proxy.hasErrors:
                raise GliderRuntimeError(command_proxy.errorMessage)
            return command_proxy
        finally:
            if command_proxy:
                command_proxy.stop()

    def execute_command_async(self, command_dataset: GliderCommandDataset) -> GliderCommandProxy:
        rsp = requests.post('{}/api/command'.format(self.url),
                            params={'command': command_dataset.get_command_name()},
                            json={'parameters': dataclasses.asdict(command_dataset)})
        rsp.raise_for_status()
        msg = rsp.json()
        if msg['level'] == 'error':
            raise GliderRuntimeError(msg['message'])
        return GliderCommandProxy(command_id=msg['command_id'],
                                  glider=self,
                                  dataset=command_dataset)

    def get_last_command_with_results(self):
        rsp = requests.get('{}/api/command/last_result'.format(self.url))
        rsp.raise_for_status()
        return GliderCommandStatus.from_dict(mydict=rsp.json(),
                                             DataSetKlass=RESULTS_COMMANDS[rsp.json()['name']])

    def get_results(self, command_status, result_indexes):
        rsp = requests.get('{}/api/results/{}'.format(self.url,
                                                              command_status.id),
                           params={'ids': result_indexes})
        rsp.raise_for_status()
        return [command_status.parameters.get_results_klass()(**r) for r in rsp.json()]


# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"


import importlib
import pkgutil
import pytest

def walk_packages(package_name):
    """Yield all modules in the given package."""
    package = importlib.import_module(package_name)
    path = package.__path__  # noqa: SLF001
    for module_info in pkgutil.walk_packages(path, package.__name__ + "."):
        yield module_info.name

@pytest.mark.parametrize("module_name", list(walk_packages("glider_client")))
def test_import_module(module_name):
    """Test that each module in glider_client can be imported."""
    importlib.import_module(module_name)


# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import os.path
from argparse import ArgumentParser

from matplotlib import pyplot
import numpy


def run():
    parser = ArgumentParser()
    parser.add_argument('files', nargs='+', type=str,
                        help='csv files containing the stepping POI')

    args = parser.parse_args()
    for fp in args.files:
        if not os.path.exists(fp):
            parser.error('file {} does not exist'.format(fp))
    stepping_poi_plot(args.files)

def stepping_poi_plot(files):
    for fp in files:
        n = numpy.loadtxt(fp, delimiter=',', usecols=[0, 1, 2, 3, 6])
        wn = n[:, 0]
        analog1 = n[:, 1]/n[:, 3]
        analog2 = n[:, 2]/n[:, 3]
        pyplot.plot(wn, analog1, label=os.path.basename(fp))
        pyplot.plot(wn, analog2 , label=os.path.basename(fp))
        has_errors = [bool(x) for x in n[:, 4]]
        if [x for x in has_errors if x]:
            pyplot.plot(wn[has_errors], analog1[has_errors], 'rx')
            pyplot.plot(wn[has_errors], analog2[has_errors], 'rx')


    pyplot.legend()
    pyplot.show()


if __name__ == '__main__':
    run()







# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import os
import re
import shutil


def rewrite_imports(base_dir):
    pattern = re.compile(r'from\s+glider\.client|import\s+glider\.client')
    replacement = lambda line: line.replace('glider.client', 'glider_client')

    for root, _, files in os.walk(base_dir):
        for file in files:
            if file.endswith(".py"):
                path = os.path.join(root, file)
                with open(path, "r", encoding="utf-8") as f:
                    lines = f.readlines()

                new_lines = [replacement(line) if pattern.search(line) else line for line in lines]

                if lines != new_lines:
                    with open(path, "w", encoding="utf-8") as f:
                        f.writelines(new_lines)
                    print(f"Updated: {path}")


def import_glider_client(glider_client_folder):
    shutil.rmtree(glider_client_folder, ignore_errors=True)
    print(f"Deleted: {glider_client_folder}")
    shutil.copytree("../glider/glider/client", glider_client_folder)
    rewrite_imports("glider_client")


if __name__ == "__main__":
    import_glider_client("glider_client")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on October 29, 2024

Copyright Alpes Lasers SA, Neuchatel, Switzerland, 2024

@author: chiesa
"""

from setuptools import setup

setup(
    setup_requires=['pbr'],
    pbr=True,
    test_suite = "glider_client.tests"
)


# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

ANALOG_CHANNEL1 = 1
ANALOG_CHANNEL2 = 2

ADC_SAMPLING_TIME_9NS = 9
ADC_SAMPLING_TIME_16NS = 16
ADC_SAMPLING_TIME_53NS = 53
ADC_SAMPLING_TIME_103NS = 103
ADC_SAMPLING_TIME_222NS = 222
ADC_SAMPLING_TIME_403NS = 403
ADC_SAMPLING_TIME_2422NS = 2422
ADC_SAMPLING_TIME_5066NS = 5066

ADC_SAMPLING_TIMES = [ADC_SAMPLING_TIME_9NS, ADC_SAMPLING_TIME_16NS, ADC_SAMPLING_TIME_53NS,
                      ADC_SAMPLING_TIME_103NS, ADC_SAMPLING_TIME_222NS, ADC_SAMPLING_TIME_403NS,
                      ADC_SAMPLING_TIME_2422NS, ADC_SAMPLING_TIME_5066NS]

ADC_SAMPLING_TIME_DICT = {k: v for k, v in enumerate(ADC_SAMPLING_TIMES)}
ADC_SAMPLING_TIME_DICT_INV = {v: k for k, v in ADC_SAMPLING_TIME_DICT.items()}


ADC_SAMPLING_TIMES = [9, 16, 53, 103, 222, 403, 2422, 5066]



import getpass
import requests
import json

SSRV_URL = 'https://ssrv/'

def ensure_cavity_system_sample(serial_number):
    cavity_name = 'GLIDER_{}'.format(serial_number)
    rsp = requests.get(SSRV_URL + 'samples',
        params={'filters': json.dumps([{'name': 'name',
        'op': 'like',
        'val': cavity_name}])
                },
                       verify=False)
    rsp.raise_for_status()
    data = rsp.json()
    if data['num_results'] == 0:
        rsp = requests.post(SSRV_URL + 'samples',
                            params={'name': 'GLIDER_{}'.format(serial_number),
                                    'fullname': 'GLIDER_{}'.format(serial_number),
                                    'sample_type': 'CAVITY_SYSTEM'},
                            verify=False)
        rsp.raise_for_status()

def get_laser_name_from_sn(sn):
    rsp = requests.get(SSRV_URL + '/results/resources/laspar',
                       data=json.dumps({'filters': json.dumps([{'name': 'no',
                                                     'op': 'eq',
                                                     'val': sn}])}),
                       headers={'Content-Type': 'application/json'},
                       verify=False)
    rsp.raise_for_status()
    data = rsp.json()
    return data['objects'][0]['data']['lasnom']


def store_stepping_poi(glider_config, dataset, comment=None):
    serial_number = glider_config.serial
    ensure_cavity_system_sample(serial_number)
    cavity_name = 'GLIDER_{}'.format(serial_number)
    laser_name = get_laser_name_from_sn(glider_config.cavities[1].laserSerial)
    laser_name2 = None
    if 2 in glider_config.cavities:
        laser_name2 = get_laser_name_from_sn(glider_config.cavities[2].laserSerial)
    dataset['laser_name'] = laser_name
    dataset['laser_name2'] = laser_name2
    dataset['poi_list_postdwellms'] = [x['postDwellMs'] for x in dataset['poi']]
    dataset['poi_list_laserdwellms'] = [x['laserDwellMs'] for x in dataset['poi']]
    dataset['poi_list_wavenumber'] = [x['wavenumber'] for x in dataset['poi']]
    dataset.pop('poi')
    rsp = requests.post(SSRV_URL + '/measures',
                        params={'sample_name': cavity_name,
                                'session_type': 'test',
                                'measure_type': 'gli_stepping_poi',
                                'station_name': 'PRISM',
                                'setup_name': None,
                                'user_name': getpass.getuser()},
                        json={'data': dataset},
                        verify=False)
    rsp.raise_for_status()
    if comment:
        meas_id = rsp.json()['catalog_id']
        rsp = requests.patch(SSRV_URL + '/measures/{}'.format(meas_id),
                             params={'operation': 'update',
                                     'user_name': getpass.getuser()},
                             json={'patch': {'comment': comment}},
                             verify=False)
        rsp.raise_for_status()


def store_adc_delay(glider_config, dataset):
    serial_number = glider_config.serial
    ensure_cavity_system_sample(serial_number)
    cavity_name = 'GLIDER_{}'.format(serial_number)
    laser_name = get_laser_name_from_sn(glider_config.cavities[int(dataset['cavity'])].laserSerial)
    dataset['laser_name'] = laser_name
    rsp = requests.post(SSRV_URL +'/measures',
                        params={'sample_name': cavity_name,
                                'session_type': 'test',
                                'measure_type': 'gli_scan_adc_delay',
                                'station_name': 'PRISM',
                                'setup_name': None,
                                'user_name': getpass.getuser()},
                        json={'data': dataset},
                        verify=False)
    rsp.raise_for_status()

if __name__ == '__main__':
    # ensure_cavity_system_sample(190)
    print(get_laser_name_from_sn(17030))
    #store_stepping_poi(190, {})


# -*- coding: utf-8 -*-
"""
Created by chiesa

Copyright Alpes Lasers SA, Switzerland
"""
__author__ = 'chiesa'
__copyright__ = "Copyright Alpes Lasers SA"

import re


def to_camel_case(snake_str):
    return "".join(x.capitalize() for x in snake_str.lower().split("_"))


def to_lower_camel_case(snake_str):
    # We capitalize the first letter of each component except the first one
    # with the 'capitalize' method and join them together.
    camel_string = to_camel_case(snake_str)
    return snake_str[0].lower() + camel_string[1:]


def CamelCase_to_snake_case(name):
    return re.sub(r'(?<!^)(?=[A-Z])', '_', name).lower()

if __name__ == '__main__':
    print(CamelCase_to_snake_case('AdminS2WriteParameters'))